//
//  Copyright © 2020 Imou. All rights reserved.
//

#import "LCVideotapePersenter.h"
#import "LCVideotapeListCell.h"
#import "LCVideotapeListHeardView.h"
#import "LCDeviceVideotapePlayManager.h"
#import <MBProgressHUD/MBProgressHUD.h>

@interface LCVideotapePersenter ()

///进行分组后的云录像数组
@property (nonatomic, strong) NSMutableArray<NSMutableArray *> *groupCloudVideos;

///进行分组后的本地录像数组
@property (nonatomic, strong) NSMutableArray<NSMutableArray *> *groupLocalVideos;

///选中的云录像数组
@property (nonatomic, strong) NSMutableSet *selectedCloudVideoSet;

///选中的本地录像数组
@property (nonatomic, strong) NSMutableSet *selectedLocalVideoSet;


@end

@implementation LCVideotapePersenter

#pragma mark - 数据获取

- (void)refreshCloudVideoListWithDate:(NSDate *)date {
    [LCProgressHUD showHudOnLowerView:nil];
    weakSelf(self);
    if (date) {
        self.currentDate = date;
    } else {
        
    }
    
    [self willChangeValueForKey:@"cloudVideoArray"];
    [self.cloudVideoArray removeAllObjects];
    [self.groupCloudVideos removeAllObjects];
    [self didChangeValueForKey:@"cloudVideoArray"];
    NSDateFormatter * dataFormatter = [[NSDateFormatter alloc] init];
    dataFormatter.dateFormat = @"yyyy-MM-dd";
    //开始时间
    NSString * startStr = [NSString stringWithFormat:@"%@ 00:00:00",[dataFormatter stringFromDate:self.currentDate]];
    //结束时间
    NSString * endStr = [NSString stringWithFormat:@"%@ 23:59:59",[dataFormatter stringFromDate:self.currentDate]];
    
    NSDateFormatter * tDataFormatter = [[NSDateFormatter alloc] init];
    tDataFormatter.dateFormat = @"yyyy-MM-dd HH:mm:ss";
    NSTimeInterval beginTime = [[tDataFormatter dateFromString:startStr] timeIntervalSince1970];
    NSTimeInterval endTime = [[tDataFormatter dateFromString:endStr] timeIntervalSince1970];
    [LCVideotapeInterface getCloudRecordsForDevice:self.videoManager.currentDevice.deviceId productId:self.videoManager.currentDevice.productId channelId:self.videoManager.currentChannelInfo.channelId beginTime:beginTime endTime:endTime Count:30 success:^(NSMutableArray<LCCloudVideotapeInfo *> * _Nonnull videos) {
        [weakself willChangeValueForKey:@"cloudVideoArray"];
        [weakself.cloudVideoArray removeAllObjects];
        [weakself addNewCloudVideos:videos];
        weakself.groupCloudVideos = [self groupVideoListWith:weakself.cloudVideoArray];
        
        if (videos.count == 0) {
            if ([self.videoManager.currentChannelInfo.storageStrategyStatus isEqualToString:@"notExist"]) {
                 [weakself.videoListPage.cloudVideoList lc_setEmyptImageName:@"common_pic_novideotape" andDescription:@"device_manager_no_cloud_storage".lc_T];
            } else {
                [weakself.videoListPage.cloudVideoList lc_setEmyptImageName:@"common_pic_novideotape" andDescription:@"video_module_none_record".lc_T];
            }
        } else {
            [weakself.videoListPage.cloudVideoList lc_setEmyptImageName:@"common_pic_novideotape" andDescription:@""];
        }
        
        [weakself didChangeValueForKey:@"cloudVideoArray"];
        
        [LCProgressHUD hideAllHuds:nil];
        [weakself.videoListPage.cloudVideoList.mj_header endRefreshing];
        [weakself.videoListPage.cloudVideoList.mj_footer resetNoMoreData];
    } failure:^(LCError * _Nonnull error) {
        [LCProgressHUD hideAllHuds:nil];
        [LCProgressHUD showMsg:error.errorMessage];
        [weakself.videoListPage.cloudVideoList lc_setEmyptImageName:@"common_pic_novideotape" andDescription:error.errorMessage];
        [weakself.videoListPage.cloudVideoList.mj_header endRefreshing];
    }];
}

- (void)refreshLocalVideoListWithDate:(NSDate *)date {
    [LCProgressHUD showHudOnLowerView:nil];
    weakSelf(self);
    if (date) {
        self.currentDate = date;
    } else {

    }

    [self willChangeValueForKey:@"localVideoArray"];
    [self.localVideoArray removeAllObjects];
    [self.groupLocalVideos removeAllObjects];
    [self didChangeValueForKey:@"localVideoArray"];

    [LCVideotapeInterface queryLocalRecordsForDevice:self.videoManager.currentDevice.deviceId productId:self.videoManager.currentDevice.productId channelId:self.videoManager.currentChannelInfo.channelId day:self.currentDate From:1 To:30 success:^(NSMutableArray<LCLocalVideotapeInfo *> *_Nonnull videos) {
        [weakself willChangeValueForKey:@"localVideoArray"];
        [weakself.localVideoArray removeAllObjects];
        weakself.localVideoArray = videos;
        self.groupLocalVideos = [self groupVideoListWith:weakself.localVideoArray];
        [weakself.videoListPage.localVideoList lc_setEmyptImageName:@"common_pic_novideotape" andDescription:videos.count==0?@"video_module_none_record".lc_T:@""];
        [weakself didChangeValueForKey:@"localVideoArray"];

        [LCProgressHUD hideAllHuds:nil];
        [self.videoListPage.localVideoList.mj_header endRefreshing];
    } failure:^(LCError *_Nonnull error) {
        [LCProgressHUD hideAllHuds:nil];
        [LCProgressHUD showMsg:error.errorMessage];
        [self.videoListPage.localVideoList lc_setEmyptImageName:@"common_pic_novideotape" andDescription:error.errorMessage];
        [self.videoListPage.localVideoList.mj_header endRefreshing];
    }];
}

- (void)loadMoreCloudVideoListWithDate:(NSDate *)date {
    [LCProgressHUD showHudOnLowerView:nil];
    weakSelf(self);
    if (date) {
        self.currentDate = date;
    }
    
    NSDateFormatter * dataFormatter = [[NSDateFormatter alloc] init];
    dataFormatter.dateFormat = @"yyyy-MM-dd";
    //开始时间
    NSString * startStr = [NSString stringWithFormat:@"%@ 00:00:00",[dataFormatter stringFromDate:self.currentDate]];
    
    NSString *lastCloudEndStr = self.cloudVideoArray.lastObject.beginTime;
    NSDateFormatter * tDataFormatter = [[NSDateFormatter alloc] init];
    tDataFormatter.dateFormat = @"yyyy-MM-dd HH:mm:ss";
    NSTimeInterval endTime = 0.0;
    if (lastCloudEndStr != nil) {
        endTime = [[tDataFormatter dateFromString:lastCloudEndStr] timeIntervalSince1970];
    }
    if (endTime == 0.0) {
        //结束时间
        NSString * endStr = [NSString stringWithFormat:@"%@ 23:59:59",[dataFormatter stringFromDate:self.currentDate]];
        endTime = [[tDataFormatter dateFromString:endStr] timeIntervalSince1970];
    }else {
        endTime = endTime - 1;
    }
    
    NSTimeInterval beginTime = [[tDataFormatter dateFromString:startStr] timeIntervalSince1970];
    [LCVideotapeInterface getCloudRecordsForDevice:self.videoManager.currentDevice.deviceId productId:self.videoManager.currentDevice.productId channelId:self.videoManager.currentChannelInfo.channelId beginTime:beginTime endTime:endTime Count:30 success:^(NSMutableArray<LCCloudVideotapeInfo *> * _Nonnull videos) {
        [weakself willChangeValueForKey:@"cloudVideoArray"];
        [weakself addNewCloudVideos:videos];
        self.groupCloudVideos = [self groupVideoListWith:weakself.cloudVideoArray];
        [weakself didChangeValueForKey:@"cloudVideoArray"];
        
        [LCProgressHUD hideAllHuds:nil];
        if (videos.count < 30) {
            [weakself.videoListPage.cloudVideoList.mj_footer endRefreshingWithNoMoreData];
        }else {
            [weakself.videoListPage.cloudVideoList.mj_footer endRefreshing];
        }
    } failure:^(LCError * _Nonnull error) {
        [LCProgressHUD hideAllHuds:nil];
        [LCProgressHUD showMsg:error.errorMessage];
        [weakself.videoListPage.cloudVideoList.mj_footer endRefreshing];
    }];
}

//- (void)loadMoreLocalVideoListWithDate:(NSDate *)date {
//    [LCProgressHUD showHudOnLowerView:nil];
//    weakSelf(self);
//    if (date) {
//        self.currentDate = date;
//    }
//    [LCVideotapeInterface queryLocalRecordsForDevice:self.videoManager.currentDevice.deviceId productId:self.videoManager.currentDevice.productId channelId:self.videoManager.currentChannelInfo.channelId day:self.currentDate From:(int)(self.localVideoArray.count + 1) To:(int)(self.localVideoArray.count + 30) success:^(NSMutableArray<LCLocalVideotapeInfo *> *_Nonnull videos) {
//        [weakself willChangeValueForKey:@"localVideoArray"];
//        [weakself.localVideoArray addObjectsFromArray:videos];
//        self.groupLocalVideos = [self groupVideoListWith:weakself.localVideoArray];
//        [weakself didChangeValueForKey:@"localVideoArray"];
//
//        [LCProgressHUD hideAllHuds:nil];
//        [self.videoListPage.localVideoList.mj_footer endRefreshing];
//        if (videos.count < 30) {
//            [self.videoListPage.localVideoList.mj_footer setState:MJRefreshStateNoMoreData];
//        }
//    } failure:^(LCError *_Nonnull error) {
//        [LCProgressHUD hideAllHuds:nil];
//        [LCProgressHUD showMsg:error.errorMessage];
//        [self.videoListPage.localVideoList.mj_footer endRefreshing];
//    }];
//}

- (void)loadMoreLocalVideoListWithDate:(NSDate *)date {
    [LCProgressHUD showHudOnLowerView:nil];
    weakSelf(self);
    if (date) {
        self.currentDate = date;
    }
    
    NSDateFormatter * dataFormatter = [[NSDateFormatter alloc] init];
    dataFormatter.dateFormat = @"yyyy-MM-dd";
    
    //开始时间
    NSString * startStr = [NSString stringWithFormat:@"%@ 00:00:00",[dataFormatter stringFromDate:date]];
    //1 首先查询列表中是否有数据，有将列表中的结束时间作为下次查询的开始时间
    if(weakself.localVideoArray.count!=0){
        NSDateFormatter * tempDateFormatter = [[NSDateFormatter alloc] init];
        tempDateFormatter.dateFormat = @"yyyy-MM-dd HH:mm:ss";
        NSDate* tempDate = [tempDateFormatter dateFromString:weakself.localVideoArray.lastObject.endTime];
        
        startStr= [tempDateFormatter stringFromDate:[NSDate dateWithTimeInterval:1 sinceDate:tempDate]];
    }
    
    NSString * endStr = @"";
    if (![[NSCalendar currentCalendar] isDateInToday:date]) {
    //如果不是今天
        endStr = [NSString stringWithFormat:@"%@ 23:59:59",[dataFormatter stringFromDate:date]];
    }else{
        //否则搜索时间为当前时间
        NSDateFormatter * dataFormatterEnd = [[NSDateFormatter alloc] init];
        dataFormatterEnd.dateFormat = @"yyyy-MM-dd HH:mm:ss";
        endStr = [dataFormatterEnd stringFromDate:date];
    }
    NSDateFormatter * dataFormatterEnd = [[NSDateFormatter alloc] init];
    dataFormatterEnd.dateFormat = @"yyyy-MM-dd HH:mm:ss";
    NSDate *startDate = [dataFormatter dateFromString:startStr];
    NSDate *endDate = [dataFormatter dateFromString:endStr];
    if ([startDate compare:endDate] == NSOrderedDescending) {
        endStr = startStr;
    }
    
    [LCVideotapeInterface queryLocalRecordsForDevice:self.videoManager.currentDevice.deviceId productId:self.videoManager.currentDevice.productId channelId:self.videoManager.currentChannelInfo.channelId StartTime:startStr EndTime:endStr From:(int)(self.localVideoArray.count + 1) To:(int)(self.localVideoArray.count + 30) success:^(NSMutableArray<LCLocalVideotapeInfo *> *_Nonnull videos) {
        [weakself willChangeValueForKey:@"localVideoArray"];
        [weakself.localVideoArray addObjectsFromArray:videos];
        self.groupLocalVideos = [self groupVideoListWith:weakself.localVideoArray];
        [weakself didChangeValueForKey:@"localVideoArray"];
        
        [LCProgressHUD hideAllHuds:nil];
        [self.videoListPage.localVideoList.mj_footer endRefreshing];
        if (videos.count < 30) {
            [self.videoListPage.localVideoList.mj_footer setState:MJRefreshStateNoMoreData];
        }
    } failure:^(LCError *_Nonnull error) {
        [LCProgressHUD hideAllHuds:nil];
        [LCProgressHUD showMsg:error.errorMessage];
        [self.videoListPage.localVideoList.mj_footer endRefreshing];
    }];
}

- (NSMutableArray *)cloudVideoArray {
    if (!_cloudVideoArray) {
        _cloudVideoArray = [NSMutableArray array];
    }
    return _cloudVideoArray;
}

- (NSMutableArray *)localVideoArray {
    if (!_localVideoArray) {
        _localVideoArray = [NSMutableArray array];
    }
    return _localVideoArray;
}

- (LCDeviceVideoManager *)videoManager {
    if (!_videoManager) {
        _videoManager = [LCDeviceVideoManager shareInstance];
    }
    return _videoManager;
}

- (NSDate *)currentDate {
    if (!_currentDate) {
        _currentDate = [NSDate new];
    }
    return _currentDate;
}

- (NSMutableArray<NSMutableArray *> *)groupCloudVideos {
    if (!_groupCloudVideos) {
        _groupCloudVideos = [NSMutableArray array];
    }
    return _groupCloudVideos;
}

- (NSMutableArray<NSMutableArray *> *)groupLocalVideos {
    if (!_groupLocalVideos) {
        _groupLocalVideos = [NSMutableArray array];
    }
    return _groupLocalVideos;
}

- (NSMutableSet *)selectedCloudVideoSet {
    if (!_selectedCloudVideoSet) {
        _selectedCloudVideoSet = [NSMutableSet set];
    }
    return _selectedCloudVideoSet;
}

- (NSMutableSet *)selectedLocalVideoSet {
    if (!_selectedLocalVideoSet) {
        _selectedLocalVideoSet = [NSMutableSet set];
    }
    return _selectedLocalVideoSet;
}

//将获取的数据进行分组。结果为按照时间倒序排列的二维数组
- (NSMutableArray *)groupVideoListWith:(NSMutableArray *)videos {
    // 线将数组按照倒叙排列
//    NSArray *sortedArray = [videos sortedArrayUsingComparator:^NSComparisonResult (id _Nonnull obj1, id _Nonnull obj2) {
//        return [[obj2 valueForKey:@"beginDate"] compare:[obj1 valueForKey:@"beginDate"]];
//    }];
    NSArray *sortedArray = videos;
    NSMutableArray *result = [NSMutableArray array];
    if ([videos.firstObject isKindOfClass:NSClassFromString(@"LCCloudVideotapeInfo")]) {
        //云录像倒序

        for (int a = 23; a >= 0; a--) {
            NSMutableArray *tempAry = [NSMutableArray array];
            for (int b = 0; b < sortedArray.count; b++) {
                id temp = sortedArray[b];
                NSDate *beginTime = [temp valueForKey:@"beginDate"];
                if (beginTime.hour == a) {
                    [tempAry addObject:temp];
                }
            }
            if (tempAry.count != 0) {
                [result addObject:tempAry];
            }
        }
    } else {
        //本地录像正序
        for (int a = 0; a < 24; a++) {
            NSMutableArray *tempAry = [NSMutableArray array];
            for (int b = 0; b < sortedArray.count; b++) {
                id temp = sortedArray[b];
                NSDate *beginTime = [temp valueForKey:@"beginDate"];
                if (beginTime.hour == a) {
                    [tempAry addObject:temp];
                }
            }
            if (tempAry.count != 0) {
                [result addObject:tempAry];
            }
        }
    }
    //便利数组

    return result;
}

- (void)addNewCloudVideos:(NSArray<LCCloudVideotapeInfo *> *)videos
{
    NSMutableArray<LCCloudVideotapeInfo *> *newVideos = @[].mutableCopy;
    LCCloudVideotapeInfo *firstCloud;
    if (self.cloudVideoArray.count > 0) {
        firstCloud = [self.cloudVideoArray lastObject];
    }
    for (int i = 0; i < videos.count; i++) {
        LCCloudVideotapeInfo *current = videos[i];
        if (firstCloud != nil && [firstCloud.beginTime isEqualToString: current.beginTime]) {
            [self.cloudVideoArray removeLastObject];
        }
        if (i < videos.count - 1) {
            LCCloudVideotapeInfo *next = videos[i + 1];
            if ([current.beginTime isEqualToString: next.beginTime]) {
                continue;
            }
        }
        [newVideos addObject:current];
    }
    [self.cloudVideoArray addObjectsFromArray:newVideos];
}

#pragma mark - scrollView代理相关,滚动到哪个页面，页面自动刷新（如果该页面数据为空）
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    if ([scrollView isKindOfClass:[UICollectionView class]]) {
        return;
    }
    if (scrollView.contentOffset.x == 0) {
        //滚动到云视频
        self.isCloudMode = YES;
        [self.videoListPage.segment setSelectIndex:0];
        [self refreshCloudVideoListWithDate:nil];
    } else {
        self.isCloudMode = NO;
        [self.videoListPage.segment setSelectIndex:1];
        [self refreshLocalVideoListWithDate:nil];
    }
}
#pragma mark - 删除数据
- (void)deleteCloudViewotape {
    weakSelf(self);
    if (self.selectedCloudVideoSet.count == 0) {
        
        [LCProgressHUD hideAllHuds:nil];
        //重新生成数据
        weakself.groupCloudVideos = [self groupVideoListWith:self.cloudVideoArray];
        [self.videoListPage.cloudVideoList reloadData];
        return;
    }
    
    if ([MBProgressHUD HUDForView:[LCProgressHUD keyWindow]] == nil) {
        [LCProgressHUD showHudOnView:nil];
    }
    
    
    NSArray<LCCloudVideotapeInfo *> * items = [self.selectedCloudVideoSet allObjects];
    [LCVideotapeInterface deleteCloudRecords:items.firstObject.recordRegionId productId: items.firstObject.productId success:^{
        [weakself.selectedCloudVideoSet removeObject:items.firstObject];
        //[self.videoListPage.cloudVideoList deleteItemsAtIndexPaths:@[items.firstObject.index]];
        
        [weakself willChangeValueForKey:@"cloudVideoArray"];
        NSMutableArray *tempArr = [NSMutableArray arrayWithArray:weakself.cloudVideoArray];
        [tempArr enumerateObjectsWithOptions:NSEnumerationReverse usingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            if ([obj isKindOfClass:[LCCloudVideotapeInfo class]]) {
                if ([((LCCloudVideotapeInfo *)obj).recordId isEqualToString:items.firstObject.recordId]) {
                    [tempArr removeObject:obj];
                }
            }
            *stop = NO;
        }];
        weakself.cloudVideoArray = tempArr;
        [weakself didChangeValueForKey:@"cloudVideoArray"];
        
        [weakself deleteCloudViewotape];
    } failure:^(LCError * _Nonnull error) {
        [LCProgressHUD hideAllHuds:nil];
        [LCProgressHUD showMsg:error.errorMessage];
    }];
    
}

#pragma mark - collection代理相关
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if (collectionView.tag == 1000) {
        //云录像
        return self.groupCloudVideos[section].count;
    } else {
        //本地录像
        return self.groupLocalVideos[section].count;
    }
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    if (collectionView.tag == 1000) {
        //云录像
        return self.groupCloudVideos.count;
    } else {
        //本地录像
        return self.groupLocalVideos.count;
    }
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    LCVideotapeListCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"LCVideotapeListCell" forIndexPath:indexPath];
    
    if (collectionView.tag == 1000) {
        //云录像
        LCCloudVideotapeInfo *info = self.groupCloudVideos[indexPath.section][indexPath.item];
        info.index = indexPath;
        cell.model = info;
    } else {
        //本地录像
        LCLocalVideotapeInfo *info = self.groupLocalVideos[indexPath.section][indexPath.item];
        cell.model = info;
    }
    
    cell.selectImg.hidden = ![self isCellChecked:indexPath];
    
    [cell.KVOController observe:self keyPath:@"isSelectAll" options:NSKeyValueObservingOptionNew block:^(id _Nullable observer, id _Nonnull object, NSDictionary<NSString *, id> *_Nonnull change) {
        if ([change[@"new"] boolValue]) {
            cell.selectImg.hidden = NO;
            if (collectionView.tag == 1000) {
                [self.selectedCloudVideoSet addObject:cell.model];
            } else {
                [self.selectedLocalVideoSet addObject:cell.model];
            }
        } else {
            cell.selectImg.hidden = YES;
            if (collectionView.tag == 1000) {
                [self.selectedCloudVideoSet removeAllObjects];
            } else {
                [self.selectedLocalVideoSet removeAllObjects];
            }
        }
    }];

    [cell.KVOController observe:self keyPath:@"isEdit" options:NSKeyValueObservingOptionNew block:^(id _Nullable observer, id _Nonnull object, NSDictionary<NSString *, id> *_Nonnull change) {
        if (![change[@"new"] boolValue]) {
            cell.selectImg.hidden = YES;
        }
    }];

    return cell;
}


- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath {
    LCVideotapeListHeardView *heardView = [collectionView dequeueReusableSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"LCVideotapeListHeardView" forIndexPath:indexPath];
    heardView.backgroundColor = [UIColor lccolor_c54];
    heardView.index = indexPath.section;
    if (collectionView.tag == 1000) {
        LCCloudVideotapeInfo *info = self.groupCloudVideos[indexPath.section][0];
        heardView.time = [NSString stringWithFormat:@"%02ld:00", info.beginDate.hour];
    } else {
        LCLocalVideotapeInfo *info = self.groupLocalVideos[indexPath.section][0];
        heardView.time = [NSString stringWithFormat:@"%02ld:00", info.beginDate.hour];
    }
    return heardView;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout referenceSizeForHeaderInSection:(NSInteger)section {
    if (section == 0) {
        CGSize size = { SCREEN_WIDTH, 27 };
        return size;
    } else {
        CGSize size = { SCREEN_WIDTH, 32 };
        return size;
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    LCVideotapeListCell *cell = (LCVideotapeListCell *)[collectionView cellForItemAtIndexPath:indexPath];
    if (self.isEdit) {
        //编辑模式状态
        cell.selectImg.hidden = !cell.selectImg.hidden;
        if (collectionView.tag == 1000) {
            if (cell.selectImg.hidden) {
                [self.selectedCloudVideoSet removeObject:cell.model];
            } else {
                [self.selectedCloudVideoSet addObject:cell.model];
            }
        } else {
            if (cell.selectImg.hidden) {
                [self.selectedLocalVideoSet removeObject:cell.model];
            } else {
                [self.selectedLocalVideoSet addObject:cell.model];
            }
        }
    } else {
        if (collectionView.tag == 1000) {
            [LCDeviceVideotapePlayManager manager].cloudVideotapeInfo = (LCCloudVideotapeInfo *)cell.model;
        } else {
            [LCDeviceVideotapePlayManager manager].localVideotapeInfo = (LCLocalVideotapeInfo *)cell.model;
        }
        [self.videoListPage.navigationController pushToVideotapePlay];
    }
}

- (BOOL)isCellChecked:(NSIndexPath *)indexPath {
    if (self.selectedCloudVideoSet.count == 0) {
        return NO;
    }
    for (LCCloudVideotapeInfo *info in self.selectedCloudVideoSet) {
        if ([info.index compare:indexPath] == NSOrderedSame) {
            return YES;
        }
    }
    return NO;
}

- (void)setIsEdit:(BOOL)isEdit {
    _isEdit = isEdit;
}

@end
